// IRNicSPA Module
// Jeffrey J Weimer
// developed from utilities created by by Don Lavery
// SVN History: $Revision: 870 $ on $Date: 2008-07-22 14:54:27 -0500 (Tue, 22 Jul 2008) $

#pragma rtGlobals=1
#pragma IgorVersion=6.03
#pragma version = 1.0

#pragma ModuleName=IRGramsSPC

//***********************************************************************************************************
//***********                                        LoadIRAction                                              ************
//*********** is the SPC File Reader.  It reads a single SPC spectrum file specified    ************
//*********** by the string, Name, and stores it in the current data folder                     ***********
//***********************************************************************************************************
 
Static Function/S LoadGramsSPC(ic,udFL)
	STRUCT udFLStandardStructure &udFL
	variable ic
  	
  	// The following duplicate the .SPC structure definitions but have been declared as individual variables
  	//	in order to avoid problems due to the differences in C and Igor structure features (e.g. string length limitations
  	//		and zero padding)
  	//*******SPCHDR Structure Variables********************************************
     	variable 	ftflgs;		// BYTE		Flag bits defined below */
      variable 	fversn;		// BYTE 		0x4B=> new LSB 1st, 0x4C=> new MSB 1st, 0x4D=> old format */
      variable 	fexper;		// BYTE		Instrument technique code (see below) */
      variable 	fexp; 		// char 		Fraction scaling exponent integer (80h=>float) */
   	variable 	fnpts;		// DWORD   	Integer number of points (or TXYXYS directory position) */
	variable 	ffirst;		// double 	Floating X coordinate of first point */
    	variable 	flast;		// double 	Floating X coordinate of last point */
    	variable 	fnsub;		// DWORD  	Integer number of subfiles (1 if not TMULTI) */
    	variable 	fxtype;		// BYTE   	Type of X axis units (see definitions below) */
   	variable 	fytype;		// BYTE    	Type of Y axis units (see definitions below) */
   	variable 	fztype;		// BYTE    	Type of Z axis units (see definitions below) */
   	variable 	fpost;		// BYTE    	Posting disposition (see GRAMSDDE.H) */
   	variable 	fdate;		// DWORD   	Date/Time LSB: min=6b,hour=5b,day=5b,month=4b,year=12b */
   	string 	fres;			// char[9]    	Resolution description text (null terminated) */
   	string 	fsource;		// char[9]    	Source instrument description text (null terminated) */
  	variable 	fpeakpt;		// WORD    	Peak point number for interferograms (0=not known) */
   	string 	fspare;		// float[8]   	Used for Array Basic storage */  *****SPCHDR calls this a float but indicates 8 bytes.  String seems more reasonable
   	string 	fcmnt;		// char[130]   Null terminated comment ASCII text string */
   	string 	fcatxt;		// char[30]    	X,Y,Z axis label strings if ftflgs=TALABS */
   	variable 	flogoff;		// DWORD  	File offset to log block or 0 (see above) */
   	variable 	fmods;		// DWORD   	File Modification Flags (see below: 1=A,2=B,4=C,8=D..) */
   	variable 	fprocs;		// BYTE    	Processing code (see GRAMSDDE.H) */
   	variable 	flevel;		// BYTE    	Calibration level plus one (1 = not calibration data) */
  	variable 	fsampin;		// WORD    	Sub-method sample injection number (1 = first or only ) */
 	variable 	ffactor;		//  float   	Floating data multiplier concentration factor (IEEE-32) */
   	variable 	fmethod;		// char[48]	Method/program/data filename w/extensions comma list */
   	variable 	fzinc;		// float  		Z subfile increment (0 = use 1st subnext-subfirst) */
   	variable 	fwplanes;	// DWORD	Number of planes for 4D with W dimension (0=normal) */
   	variable 	fwinc;		// float   		W plane increment (only if fwplanes is not 0) */
   	variable 	fwtype;		// BYTE    	Type of W axis units (see definitions below) */
   	string 	freserv; 		// char[187]   Reserved (must be set to zero) */
   	//*********End SPCHDR;**************************************************************

  	//**********SUBHDR Structure Variables*********************************************
   	variable subflgs;		// BYTE   	Flags as defined above */
   	variable subexp;		// char  		Exponent for sub-file's Y values (80h=>float) */
   	variable subindx;		// WORD  	Integer index number of trace subfile (0=first) */
   	variable subtime;		// float  		Floating time for trace (Z axis coordinate) */
  	variable subnext;		// float  		Floating time for next trace (May be same as beg) */
   	variable subnois;		// float  		Floating peak pick noise level if high byte nonzero */
   	variable subnpts;		// DWORD  	Integer number of subfile points for TXYXYS type */
   	variable subscan;		// DWORD  	Integer number of co-added scans or 0 (for collect) */
  	variable subwlevel;	//  float  		Floating W axis value (if fwplanes non-zero) */
   	variable subresv;		// char[4]  	Reserved area (must be set to zero) */
   	//********End SUBHDR;*****************************************************************
   
   	//********LOGSTC Structure Variables**************************************************
  	//******** log block header format *******************************************************
  	variable 	logsizd;		// DWORD	byte size of disk block */
   	variable 	logsizm;		// DWORD  	byte size of memory block */
   	variable 	logtxto;		// DWORD  	byte offset to text */
   	variable 	logbins;		// DWORD  	byte size of binary area (immediately after logstc) */
   	variable 	logdsks;		// DWORD  	byte size of disk area (immediately after logbins) */
   	string	logspar;		// char[44]  	reserved (must be zero) */
   	//*********End LOGSTC;******************************************************************
	
	//*********LOGTEXT BLOCK*************************************************************
	string logstr			// char[variable]	a null terminated string of variable length in keyword form
	//*******************************************************************************************
	
  	//	Auxiliary variables used to build an Igor wave from the spectral data
  	Variable FileVar
  	String PathName=""
  	String SDummy=""		// Dummy string used to read non essential data
  	Variable Dummy			// Dummy variable used to read non-essential data
  	Variable ni=0
 	Variable a				// used to hold the scaling factor calculated fro fexp  a=2^(32-fexp)
	variable minute,hour,day,month,year
	string datestr,timestr
	string memo				// the Grams memo line
	string notestr				// the Igor note - a (":","\r" delimited) keyword string of all the auxiliary spectrum information

	string wList = ""
	string fileStr=StringFromList(ic,udFL.fileList)

	// read each file
		
	if (udFL.reportCtrl >= 1)
		print "... file ", fileStr
	endif
	
	if (strlen(udFL.pathStr)==0)
		Open/R FileVar fileStr
	else
		Open/R/P=tmpIRDatadir FileVar fileStr
	endif

	FBinRead/B/F=1 FileVar,ftflgs					// Read flags 
	FBinRead/B/F=1 FileVar,fversn					// File format: 75=new LSB version  ///edited dummy=>fversn DSL 
	if(fversn!=0x4B)								// exit with warning if old (LabCalc) file.  Value of fexp will allow scaled fixed point or floating point
		udFL.errCode = -1
		udFL.errMsg = "This is not a new format Grams file"
		Close FileVar
		udFLHandleError(udFL)
		return wlist
	endif

	FBinRead/B/F=1 FileVar,fexper					// experiment type e.g. 0=general, 4=FTIR
	FBinRead/B/F=1 FileVar,fexp					// Data exponent		 ****DSL swapped defiinition of fexp and a
	a=2^(32-fexp)									// Calc data exponent	 **** to keep definition of fexp consistent with spc file spec
	FBinRead/B=3/F=3 FileVar,fnpts				// # of points in spectrum
	FBinRead/B=3/F=5 FileVar,ffirst				// Scaling; start  -  64 bit IEEE 
	FBinRead/B=3/F=5 FileVar,flast				// Scaling: end  -  64 bit IEEE
	FBinRead/B=3/F=3 FileVar,fnsub				// # of sub files/waves  -  32 bit signed 
	if((fnsub!=1)&&(udFL.reportCtrl>0))								// allow multifiles but only read the first one
		udFL.errCode = 0
		udFL.errMsg = "Reading only the first subfile in multi-file " + fileStr
		udFLHandleError(udFL)
	endif
	FBinRead/B/F=1 FileVar,fxtype					// x axis units
	FBinRead/B/F=1 FileVar,fytype					// y axis units
	FBinRead/B/F=1 FileVar,fztype					// z axis units	
	FBinRead/B/F=1 FileVar,Dummy				// Posting disposition (refers to value from GramsDDE)
	FBinRead/B=3/F=3 FileVar,fdate				// Date variable  dword
	FReadLine/N=9/T=("") FileVar,fres					// resolution description 			char[9]
	FReadLine/N=9/T=("") FileVar,fsource					// data source description			char[9]
	FBinRead/B/F=2 FileVar,fpeakpt				// interferogram centerburst peak point num.  (0=not known)  16 bit 
	FReadLine/N=8 FileVar,fspare					// used for array basic storage		char[8]
	Memo=FindMemo(FileVar)						// find any non-null string in the memo area - null string if nothing there
	
	FSetPos FileVar,194							// file pointer to byte after memo area
	FReadLine/N=30/T=("") FileVar,fcatxt			// axis labels if not standard, 30 bytes max, null terminated

	FSetPos FileVar,224							// file pointer to byte after fcatxt area
	FBinRead/B/F=3 FileVar,flogoff					// flogoff dword 	file offset to log block
	FBinRead/B/F=3 FileVar,fmods					// fmods dword	code for modifications made to spectrum after collect e.g. Trans=>Abs,baseline, etc.
	FBinRead/B/F=1 FileVar, fprocs				// fprocs			processing code (grams.dde)
	FBinRead/B/F=1 FileVar,flevel					// flevel			calibration level plus 1 (1 = not calibration data)
		
	FSetPos FileVar,512							// Start of SubFile Header, 32 bytes
	FBinRead/B/F=1 FileVar,subflgs				// subflgs
	FBinRead/B/F=1 FileVar,subexp				// subfile data exponent
	FBinRead/B/F=2 FileVar,subindx				// sub file #
	FBinRead/B/F=4 FileVar,subtime				// floating time for trace
	FBinRead/B/F=4 FileVar,subnext				// floating time for next trace
	FBinRead/B/F=4 FileVar,subnois				// floating peak pick noise level if high byte non-zero
	FBinRead/B/F=3 FileVar,subnpts				// Integer number of subfile points for TXYXYS type
	FBinRead/B/F=3 FileVar,subscan				// Integer number of co-added scans
	FBinRead/B/F=4 FileVar,subwlevel				// floating W axis value (if fwplanes non-zero)
	// subresv reserved 4 bytes must be set to zero	// don't bother to read
	if(flogoff)										// if there is no Log Data block, flogoff will be zero
		FSetPos FileVar,flogoff						// offset value (read from SPCHDR) to the Log Data Header
		FBinRead/B/F=3 FileVar, logsizd;				// DWORD	byte size of disk block */
	   	FBinRead/B/F=3 FileVar,logsizm;				// DWORD  	byte size of memory block */
	   	FBinRead/B/F=3 FileVar,logtxto;				// DWORD  	byte offset to text */
	   	FBinRead/B/F=3 FileVar,logbins;				// DWORD  	byte size of binary area (immediately after logstc) */
	   	FBinRead/B/F=3 FileVar, logdsks;				// DWORD  	byte size of disk area (immediately after logbins) */
	   	FReadLine/N=(44) FileVar,logspar;				// char[44]  	reserved (must be zero) */
		FReadLine/T=("") FileVar,logstr					// read the null terminated keyword string "keyword=value<cr><lf>"
		logstr=ReplaceString(num2char(10), logstr, "");	// convert the string to an Igor keyword string "keyword=value\r".  i.e. remove <lf>
		// the entries in this string will depend on the instrument and software used to collect the spectra.
		//	we will add it to the note so that is available within Igor.  Standard keywords are tabulated in gspc_udf.doc
	else
		logstr=""										// should be initialized to null anyway but just in case
	endif	

	//**********Now read the spectral data and build the wave 
//	Make/O/N=(fnpts) $(RemoveEnding(Name,".spc"))	// Make wave for data
//	wave IRData=$(RemoveEnding(Name,".spc"))		//	and make a convenient local wave reference

	make/O/N=(fnpts) IRData
	wave IRData = IRData

	FSetPos FileVar,544
	If(fexp==-128)								// This is floating point data 
		FBinRead/B=3/F=4 FileVar,IRData			// Load spectrum  -  32 bit IEEE		
	Else
		FBinRead/B=3/F=3 FileVar,IRData			// Load 32 bit signed word data
	 	IRData/=a								// Scale the Integer data to real values
	EndIf	

 	Close FileVar								// Close file 	

	SetScale /I x, ffirst, flast, GetxLabel(fxtype), IRData	// Scale spectrum x units according to spc fxtype code 

//	//Now check to see that the x values increase with increasing point value.  If they don't than swap
//	//all the points end for end.  (this for consistency with our other file readers  -  don't know if or when it is important)
//	If (ffirst > flast) 								//we need to flip the wave to make sure some IGOR functions won't be confused.
//		reverse IRData
//	endif
//	if(fytype==-128||fytype==-129)
//		wavestats/Q IRData
//		if(V_avg>10)								// assume values > 10 are % otherwise fractional, may have to change for really bad spectra
//			SetScale d, 0, 100, "%"+GetyUnits(fytype),IRData	// set the y units type %Transmittance or %Reflectance
//		else
//			SetScale d, 0, 100, GetyUnits(fytype),IRData		// otherwise just Transmittance or Reflectance
//		endif
//	else
//		SetScale d, 0, 100, GetyUnits(fytype),IRData			// all other y units (e.g. Absorbance), just use the units entry
//	endif

	// decode the date value
	minute=fdate&0x3F						// extract 6 bit minutes
	fdate=floor(fdate/0x40)							// right shift 6 bits (need to truncate to remove fractional part of result)
	hour=fdate&0x1F							// extract 5 bit hours
	fdate=floor(fdate/0x20)							// right shift 5
	day=fdate&0x1F							// extract 5 bit day
	fdate=floor(fdate/0x20)							// right shift 5
	month=fdate&0x0F						// extract 4 bit month
	fdate=floor(fdate/0x10)							// right shift 4
	year=fdate								// keep remaining 12 bit year
	sprintf timestr, "%02d:%02d",hour,minute		// convert hour and minute to string
	timestr+=":00"							// add 00, SPC does not record seconds
	datestr=JulianToDate(dateToJulian(year, month, day ),2)
	// make the note a keyword list
	notestr="Trace Type: "+GetExperimentType(fexper)+"\r"
	notestr+="Source: "+NoNull(fsource)+"\r"	// replace null string with space or remove terminating null so cr will not be ignored
	notestr+="Date: "+datestr+"\r"
	notestr+="Time: "+timestr+"\r"
	notestr+="#Scans: "+num2str(subscan)+"\r"
	notestr+="First x: "+num2str(ffirst)+"\r"
	notestr+="Last x: "+num2str(flast)+"\r"
	notestr+="Resolution: "+NoNull(fres)+"\r"
	notestr+="User Text: "+NoNull(memo)+"\r"
	notestr+="\r"								// separate the log data string from the preceding entries with a blank line
	notestr+= NoNull(Logstr)
	sprintf notestr, "<fileheader>\r%s\r</fileheader>" noteStr
	
	note IRData,notestr						// and put the whole thing into the wave note
	
	wlist = "IRData;"
	
	return wlist
End


Static Function/S GetxLabel(xcode)
	variable xcode
	
	switch (xcode)
		case 0:
			return "Arbitrary"
		break
		case 1:
			return "Wavenumber (cm-1)"
		break
		case 2:
			return "Wavelength (m)"
		break
		case 3:
			return "Wavelength (nm)"
		break
		case 4:
			return "Time (sec)"
		break
		case 5:
			return "Time (min)"
		break
		case 6:
			return "Frequency (Hz)"
		break
		case 255:
			return " "
		break
		default:
			return "Unknown"
		break
	endswitch	
End

Static Function/S GetxUnits(xcode)
	variable xcode
	
	switch (xcode)
		case 0:
			return " "
		break
		case 1:
			return "cm-1"
		break
		case 2:
			return "m"
		break
		case 3:
			return "nm"
		break
		case 4:
			return "sec"
		break
		case 5:
			return "min"
		break
		case 6:
			return "Hz"
		break
		case 255:
			return " "
		break
		default:
			return "Unknown"
		break
	endswitch	
End

Static Function/S GetyUnits(ycode)
	variable ycode
	
	switch (ycode)
		case 0:
			return "Arbitrary Intensity"
		break
		case 1:
			return "Interferogram"
		break
		case 2:
			return "Absorbance"
		break
		case 3:
			return "Kubelka-Munk"
		break
		case 4:
			return "Counts"
		break
		case 5:
			return "Volts"
		break
		case 6:
			return "Degrees"
		break
		case -128:
			return "Transmittance"			
		break
		case -129:
			return "Reflectance"
		default:
			return "Unknown"
		break
	endswitch	
End

Static Function/S GetExperimentType(fexper)
	Variable fexper
	
	switch(fexper)
		case 0:
			return "General"
		break
		case 4:
			return "FTIR"
		break
		case 0x0C:
			return "Fluorescence"
		break
		default:
			return "Not recognized"
		break
	endswitch
End

Static Function/S FindMemo(FileVar)				// find the first non-null string anywhere within the 130 byte spc memo area
	Variable FileVar
	
	Variable i,byteTemp,memostart,memochar
	String memo
	i=0
	FSetPos FileVar,64											// beginning of memo string according to Galactic spec
	do															// 	but see if there are any non-null char in the memo area
		FBinRead/B/F=1 FileVar,byteTemp
		i+=1
	while ((byteTemp==0)&&(i<130))
	memostart=0x40+i-1											// calc the offset to the beginning of the first non-null string
	memochar=130-i-1											// calc the length of hthe string
	if(memochar>0)
		FSetPos FileVar,memostart								// reset the offset to the beginning of the first non-null string
		FReadLine/N=(memochar)/T=("") FileVar,Memo				// memo - read null terminated string but not more than 130 char 
	else
		memo=""												// null string if search fails
	endif
	return memo
End

Static Function/S NoNull(str)					// FReadLine/T=("") reads a null terminated string and returns it including the null
	string str							// notestr+=str+"\r" will ignore the "\r" if str is null terminated
	
	if(strlen(str)==0)					// if str is null, replace it with space
		str=" "
	else
		if (char2num(str[strlen(str)-1])==0)		// is final char a null?
			str=str[0,strlen(str)-2]		// remove it
		endif		
	endif
	return str						// return the string
End							// the original string will be unchanged in the calling program unless the return value is assigned to it

