// IRNicSRS Module
// Jeffrey J Weimer
// developed from utilities created by J. Gutow
// SVN History: $Revision: 870 $ on $Date: 2008-07-22 14:54:27 -0500 (Tue, 22 Jul 2008) $

#pragma rtGlobals=1
#pragma IgorVersion=6.03
#pragma version = 1.0

#pragma ModuleName=IRNicSRS

#include "udIRModule_NicCore"

Static Function/S LoadNicSRS(ic,udFL)
	STRUCT udFLStandardStructure &udFL
	variable ic

	string wList = ""
	string fileStr=StringFromList(ic,udFL.fileList)

	variable filenumber, nspec, minperspec
	variable spec_firstx, spec_lastx, delta, spec_points, V_filetype, V_nspectra, nSec, secOffset, spec_V_nscans, spec_V_nBkgScans
	variable bac_firstx, bac_lastx,bac_points,bac_V_nscans, bac_V_nBkgScans, points
	variable a, b, c, n, secType, secLength, whichspec, sec2count
	string nameforwave, executestring, str, title, S_vers, S_date, S_backDate, S_yaxislabel, UserText, S_datatype
	string S_resolution, s_beamsplitter, TraceName

	// read each file
		
	if (udFL.reportCtrl >= 1)
		print "... file ", fileStr
	endif
	
	if (strlen(udFL.pathStr)==0)
		Open/R filenumber fileStr
	else
		Open/R/P=tmpIRDatadir filenumber fileStr
	endif
	
	FReadLine filenumber,str; S_vers=str[0,strlen(str)-2]								// data file type
	FSetPos filenumber,30; FReadLine/T="" filenumber,str; title=str[0,strlen(str)-2]		// title
	FSetPos filenumber,288; FBinRead/B=0/F=2 filenumber,V_FileType					// Should be 11
	FSetPos filenumber,292; FBinRead/B=0/F=2 filenumber,V_nspectra					// # of index sections
	FBinRead/B=0/F=2 filenumber,nSec 													// # of sections
	//print "FileType:",V_FileType," index sections:", V_nspectra,"number of sections:", nSec
	n=0
	whichspec=0
	sec2count = 0
	do
		FSetPos filenumber,304+16*n; FBinRead/B=0/F=2 filenumber,secType
		FBinRead/B=0/F=3 filenumber,secOffset
		FBinRead/B=0/F=3 filenumber,secLength
		//print "section #:",n," SecType:",SecType," SecOffset:",SecOffset," SecLength:",SecLength
		switch (secType)														// Select data type
			case 2:															// Spectral Header
				sec2count=sec2count+1
				switch(sec2count)
					case 1:													//Header for ratioed spectra
						IRNicCORE#ReadNicSection2(filenumber,secOffset, spec_points, spec_lastx, spec_firstx, spec_V_nscans, spec_V_nBkgScans) //for some reason
						//firstx and lastx are switched in .SRS files...
					break
					case 2:													//Header for background spectrum
						IRNicCORE#ReadNicSection2(filenumber,secOffset, bac_points, bac_lastx,bac_firstx, bac_V_nscans, bac_V_nBkgScans) //for some reason
						//firstx and lastx are switched in .SRS files...
					break
				endswitch
				break
			case 3:															// Spectral Data
				if (whichspec==0)     //either we're loading the initial background or one of spectral series
					points=bac_points
				else
					points=spec_points
				endif
				make /O/n=(points) DataWave
				wave DataWave = DataWave
				IRNicCORE#ReadNicSection3(filenumber, SecOffset, DataWave)
				if (whichspec==0)		//store data for spectrum in the wave note and wave scales
					nameforwave = "IRData_"+ "background"
					s_datatype = "background"
					SetScaleandNote(DataWave, bac_firstx, bac_lastx, title,TraceName, s_resolution, s_date, s_backdate, s_datatype, bac_V_nscans, bac_V_nBkgscans, UserText, S_yaxislabel)
				else
					if (whichspec < 10)		
							nameforwave="IRData_000"+num2str(whichspec) //a wave name so no extension
					endif
					if ((whichspec<100) && (whichspec >= 10)) 		
							nameforwave="IRData_00"+num2str(whichspec) //a wavename so no extension
					endif
					if ((whichspec <1000) && (whichspec >=100))						
							nameforwave="IRData_0"+num2str(whichspec) //a wavename so no extension
					endif
					if ((whichspec <10000) && (whichspec >=1000))
							nameforwave="IRData_" +num2str(whichspec)  //a wavename so no extension
					endif
					s_datatype = "spectrum"
					SetScaleandNote(DataWave, spec_firstx, spec_lastx, title,TraceName, s_resolution, s_date, s_backdate, s_datatype, spec_V_nscans, spec_V_nBkgscans, UserText, S_yaxislabel)
				endif					
				whichspec=whichspec+1 //increment the spectrum counter

				//now let's rename the wave
				duplicate/O DataWave $nameforwave
				wlist += nameforwave + ";"
				break
			case 4:															// User text
				UserText=IRNicCORE#ReadNicSection4(filenumber, secOffset)
				break
			case 27:															// History text
				IRNicCORE#ReadNicSection27(filenumber, secOffset, S_Backdate, S_datatype, S_yaxislabel, S_Resolution, S_beamsplitter)
				//print "backdate:",S_Backdate," datatype:",S_datatype," ylabel:",S_yaxislabel," resolution:",S_Resolution," beamsplitter:",S_beamsplitter
				break
			case 105:														// Parameters don't know
				IRNicCORE#ReadNicSection105(filenumber, secOffset, a,b,c)
				break
			case 106:														// Parameters don't know
			//	FSetPos file,secOffset; FBinRead/B=0/F=2 file,wave1
				break
			case 107:														//trace name info
				TraceName =IRNicCORE#ReadNicSection107(filenumber, secOffset)
				break
			case 113:														//Parameters don't know
				break
			case 130:														//Parameters don't know
				break
			case 146:														//Parameters don't know
				break
			case 148:														//tracename and delay info
				TraceName=IRNicCORE#ReadNicSection148(filenumber,secOffset)
				//print TraceName
				break
			case 151:														//number of series spectra and minutes per spectrum
				IRNicCORE#ReadNicSection151(filenumber,secOffset, nspec, minperspec)
				break
		endswitch
		n+=1
	while (n<nSec)	
	
	//Check that we loaded the right number of spectra...sometimes the files don't have the right number or the wrong data
	//may be in section information.
	if (nspec != (whichspec-1))
		udFL.errCode = 0
		udFL.errMsg = "Data file does not contain the expected number of spectra.  Was able to load " + num2str(whichspec-1) + " spectra."
		udFLHandleError(udFL)
		nspec = (whichspec-1)
	endif
		
	//Cleanup
	killwaves DataWave
	return wlist
end

Static Function SetScaleandNote(DataWave, firstx, lastx, title, TraceName, s_resolution, s_date, s_backdate, s_datatype, V_nscans, V_nBkgscans, UserText, ylabel)
	Wave Datawave
	variable firstx, lastx
	string title, TraceName, s_resolution, s_date, s_backdate, s_datatype, ylabel
	variable  V_nscans, V_nBkgscans
	string UserText
	SetScale/I x firstx,lastx, DataWave
//	SetScale d 0,100,ylabel,DataWave
	Note/K DataWave
	Note DataWave, ("Title: "+title)
	Note DataWave, ("TraceName: "+TraceName)
	Note DataWave, ("Resolution: "+S_resolution)
	Note DataWave, ("Date: "+S_date)
	Note DataWave, ("Background date: "+S_Backdate)
	Note DataWave, ("DataType:" + s_datatype)
	Note DataWave, ("Number of Scans: "+num2Str(V_nscans))
	Note DataWave, ("Number of Background Scans: "+num2Str(V_nBkgScans))
	Note DataWave, ("UserText: "+UserText)
//
//	//Now check to see that the x values increase with increasing point value.  If they don't than swap
//	//all the points end over end.
//	If (firstx > lastx) //we need to flip the wave to make sure some IGOR functions won't be confused.
//		SwapPointOrder(DataWave)
//	endif
end
